/************************************************************************
Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery and
Retrieval, 1994. 

Permission to use, copy, modify, distribute, and sell this software and
its documentation, in whole or in part, for any purpose is hereby granted
without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included in
this distribution must remain intact. 

2. Users of this software agree to make their best efforts (a) to return
to MCNC any improvements or extensions that they make, so that these may
be included in future releases; and (b) to inform MCNC/CNIDR of noteworthy
uses of this software. 

3. The names of MCNC and Clearinghouse for Networked Information Discovery
and Retrieval may not be used in any advertising or publicity relating to
the software without the specific, prior written permission of MCNC/CNIDR. 

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY
OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR
IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
************************************************************************/

/*@@@
File:		tcpstr.cxx
Version:	1.00
Description:	TCP/IP Stream class based on C++ streams.
Author:		Kevin Gamiel, Kevin.Gamiel@cnidr.org
@@@*/

#include <iostream.h>
#include "tcpstr.hxx"

inline void
tcpstreambase::__ib_init()
{
	// Initialize class
}

tcpstreambase::tcpstreambase() : iostream(&c_tcpbuf)
{
#ifdef IPDEBUG1
	cout << "tcpstreambase::tcpstreambase()\n";
#endif
	__ib_init();
}

void tcpstreambase::Open(const PCHR host, UINT port)
{
#ifdef IPDEBUG1
	cout << "tcpstreambase::open()\n";
#endif
	//clear();
	if (!rdbuf()->Open(host, port)) {
#ifdef IPDEBUG1
		cout << "tcpstreambase::open(): Failed\n";
#endif
#ifdef _MSC_VER
		// if an argument is given to clear then it acts as setstate
		iostream::clear(badbit);
#else
		iostream::setstate(badbit);
#endif
	}
}
 
void tcpstreambase::Close()
{
#ifdef IPDEBUG1
	cout << "tcpstreambase::close()\n";
#endif
	// Flush any remaining data before closing.
	// Maybe this should be optional?
	if(IsConnected())
		flush();

	if (!rdbuf()->Close())
#ifdef _MSC_VER
	  // if an argument is given to clear then it acts as setstate
	  clear(failbit);
#else
	setstate(failbit);
#endif
}
 
